@@ -186,8 +186,10 @@ static cmark_node *fixup_nodes(cmark_syntax_extension *self,
 
   if (!named_link || !named_link->ref) {
     int actual_line, actual_col;
+    cmark_node *first_parent_block = get_first_parent_block(parent);
+    char *filename = cmark_node_filename(first_parent_block);
 
-    translate_sourcepos(get_first_parent_block(parent),
+    translate_sourcepos(first_parent_block,
         start_offset, &actual_line, &actual_col);
 
     cmark_strbuf *message = cmark_strbuf_new(0);
@@ -195,7 +197,8 @@ static cmark_node *fixup_nodes(cmark_syntax_extension *self,
     cmark_strbuf_puts(message, cmark_strbuf_get(name));
     cmark_strbuf_puts(message, "’");
     diagnose("gtk-doc-bad-link", cmark_strbuf_get(message), actual_line - 1,
-        actual_col - 1);
+        actual_col - 1, filename);
+    free(filename);
     cmark_strbuf_free(message);
     cmark_node_set_literal(prev, cmark_strbuf_get(name));
     cmark_strbuf_free(name);
@@ -221,6 +224,7 @@ static cmark_node *function_link_match(cmark_syntax_extension *self,
   delimiter *tmp_delim;
   int offset;
   int start;
+  int tmp_pos;
   ParsingContext context;
 
   context.parser = inline_parser;
@@ -255,12 +259,14 @@ static cmark_node *function_link_match(cmark_syntax_extension *self,
   if (!ret)
     goto done;
 
+  tmp_pos = cmark_inline_parser_get_offset (inline_parser);
   tmp_delim = cmark_inline_parser_get_last_delimiter(inline_parser);
 
   while (tmp_delim) {
     delimiter *prev = tmp_delim->previous;
+    tmp_pos -= tmp_delim->length;
 
-    if (tmp_delim->position < start + 1) {
+    if (tmp_pos < start + 1) {
       break;
     }
 
@@ -318,16 +324,22 @@ static cmark_node *symbol_link_match(cmark_syntax_extension *self,
   char *symbol_name = NULL;
   NamedLink *named_link = NULL;
   int start_offset = cmark_inline_parser_get_offset(inline_parser);
+  char next_char = cmark_inline_parser_peek_at(
+      inline_parser,
+      start_offset + 1);
   ParsingContext context;
 
   context.parser = inline_parser;
   context.allow_dashes = 0;
 
+  /* Make sure we don't try to resolve empty links */
+  if (!next_char || next_char == ' ' || next_char == '\t' || next_char == '\n' || !is_valid_c (next_char, 0, NULL))
+    return NULL;
+
   if (start_offset > 0) {
     char prev_char = cmark_inline_parser_peek_at(
         inline_parser,
         start_offset - 1);
-
     if (prev_char && prev_char != ' ' && prev_char != '\t' && prev_char != '\n')
       return NULL;
   }
@@ -343,14 +355,17 @@ static cmark_node *symbol_link_match(cmark_syntax_extension *self,
   named_link = PRIV(self)->link_resolve_func(symbol_name);
   if (!named_link || !named_link->ref) {
     int actual_line, actual_col;
+    cmark_node *first_parent_block = get_first_parent_block(parent);
+    char *filename = cmark_node_filename(first_parent_block);
 
-    translate_sourcepos(get_first_parent_block(parent),
+    translate_sourcepos(first_parent_block,
         start_offset, &actual_line, &actual_col);
     cmark_strbuf *message = cmark_strbuf_new(0);
     cmark_strbuf_puts(message, "Trying to link to non-existing symbol ‘");
     cmark_strbuf_puts(message, symbol_name);
     cmark_strbuf_puts(message, "’");
-    diagnose("gtk-doc-bad-link", cmark_strbuf_get(message), actual_line - 1, actual_col - 1);
+    diagnose("gtk-doc-bad-link", cmark_strbuf_get(message), actual_line - 1, actual_col - 1, filename);
+    free(filename);
     cmark_strbuf_free(message);
     link = cmark_node_new (CMARK_NODE_TEXT);
     cmark_node_set_literal (link, symbol_name);
